#include <user_interface.h>
#include <osapi.h>
#include <c_types.h>
#include <mem.h>
#include <os_type.h>
#include "httpclient.h"
#include "driver/uart.h"
#include "driver/i2c.h"
#include "driver/i2c_bmp180.h"
#include "driver/i2c_sht21.h"
#include "user_config.h"

typedef enum {
    WIFI_CONNECTING,
    WIFI_CONNECTING_ERROR,
    WIFI_CONNECTED,
} tConnState;

static os_timer_t measure_timer;
LOCAL void ICACHE_FLASH_ATTR setup_wifi_st_mode(void);
static struct ip_info ipConfig;
static ETSTimer WiFiLinker;
static tConnState connState = WIFI_CONNECTING;

extern int ets_uart_printf(const char *fmt, ...);
int (*console_printf)(const char *fmt, ...) = ets_uart_printf;

const char *WiFiMode[] = {
    "NULL",		// 0x00
    "STATION",	// 0x01
    "SOFTAP", 	// 0x02
    "STATIONAP"	// 0x03
};

const char *WiFiStatus[] = {
    "STATION_IDLE", 			// 0x00
    "STATION_CONNECTING", 		// 0x01
    "STATION_WRONG_PASSWORD", 	// 0x02
    "STATION_NO_AP_FOUND", 		// 0x03
    "STATION_CONNECT_FAIL", 	// 0x04
    "STATION_GOT_IP" 			// 0x05
};


LOCAL void ICACHE_FLASH_ATTR my_http_callback(char * response, int http_status, char * full_response) {
    os_printf("Answers: \r\n");
    if (http_status == 200) {
        os_printf("response=%s\r\n", response);
        os_printf("---------------------------\r\n");
    } else {
        os_printf("http_status=%d\r\n", http_status);
        os_printf("strlen(response)=%d\r\n", strlen(response));
        os_printf("strlen(full_response)=%d\r\n", strlen(full_response));
        os_printf("response=%s\r\n", response);
        os_printf("---------------------------\r\n");
    }
}

LOCAL void ICACHE_FLASH_ATTR measure_cb(void *arg) {
    static char data[256];
    static char buff[20];
    int32_t temperature;
    int32_t pressure;
    int32_t silabsTemperature;
    int32_t silabsRH;
    console_printf("measure callback\r\n");
    os_timer_disarm(&measure_timer);
    switch(connState) {
    case WIFI_CONNECTED:
        temperature = BMP180_GetTemperature();
        pressure = BMP180_GetPressure(OSS_0) / 10;

        console_printf("BMP180 Temperature: %s *C, pressure %d.%d hPa\r\n",
                       BMP180_Int2String(buff, temperature), (int)(pressure/10), (int)(pressure%10));

        silabsTemperature = SHT21_GetVal(GET_SHT_TEMPERATURE);
        silabsRH = SHT21_GetVal(GET_SHT_HUMIDITY);
        console_printf("SHT21 temperature %d.%d *C, RH %d.%d%%\r\n",
                       silabsTemperature/10, silabsTemperature%10,
                       silabsRH/10, silabsRH%10
                       );

        wifi_get_ip_info(STATION_IF, &ipConfig);

        os_sprintf(data, "s=Temp: %d.%dC  %d.%d%%25  %dhPa",        //%25 = '%' url-encoded
                   silabsTemperature/10, silabsTemperature%10,
                   silabsRH/10, silabsRH%10,
                   (int)(pressure/10));
        http_post("http://"HTTP_SERVER"/"HTTP_TARGET, data, my_http_callback);

        break;
    default:
        os_printf("WiFi not connected...\r\n");
    }
    os_timer_setfn(&measure_timer, (os_timer_func_t *)measure_cb, (void *)0);
    os_timer_arm(&measure_timer, DATA_SEND_DELAY, 1);
}

static void ICACHE_FLASH_ATTR wifi_check_ip(void *arg) {
    os_timer_disarm(&WiFiLinker);
    switch(wifi_station_get_connect_status()) {
    case STATION_GOT_IP:
        wifi_get_ip_info(STATION_IF, &ipConfig);
        if(ipConfig.ip.addr != 0) {
            connState = WIFI_CONNECTED;
            os_printf("WiFi is connected\r\n");
        } else {
            connState = WIFI_CONNECTING_ERROR;
            os_printf("WiFi connected, ip.addr is null\r\n");
        }
        break;
    case STATION_WRONG_PASSWORD:
        connState = WIFI_CONNECTING_ERROR;
        os_printf("WiFi connecting error, wrong password\r\n");
        break;
    case STATION_NO_AP_FOUND:
        connState = WIFI_CONNECTING_ERROR;
        os_printf("WiFi connecting error, ap not found\r\n");
        break;
    case STATION_CONNECT_FAIL:
        connState = WIFI_CONNECTING_ERROR;
        os_printf("WiFi connecting fail\r\n");
        break;
    default:
        connState = WIFI_CONNECTING;
        os_printf("WiFi connecting...\r\n");
    }
    os_timer_setfn(&WiFiLinker, (os_timer_func_t *)wifi_check_ip, NULL);
    os_timer_arm(&WiFiLinker, 5000, 0);
}


LOCAL void ICACHE_FLASH_ATTR setup_wifi_st_mode(void) {
    wifi_set_opmode(STATION_MODE);
    struct station_config stconfig;
    wifi_station_disconnect();
    wifi_station_dhcpc_stop();
    if(wifi_station_get_config(&stconfig)) {
        os_memset(stconfig.ssid, 0, sizeof(stconfig.ssid));
        os_memset(stconfig.password, 0, sizeof(stconfig.password));
        os_sprintf(stconfig.ssid, "%s", WIFI_CLIENTSSID);
        os_sprintf(stconfig.password, "%s", WIFI_CLIENTPASSWORD);
        if(!wifi_station_set_config(&stconfig)) {
            os_printf("ESP8266 not set station config!\r\n");
        }
    }
    wifi_station_connect();
    wifi_station_dhcpc_start();
    wifi_station_set_auto_connect(1);
    os_printf("ESP8266 in STA mode configured.\r\n");
}

void user_rf_pre_init(void)
{
}

void user_init(void) {
    // Configure the UART
    uart_init(BIT_RATE_115200, 0);
    os_delay_us(1000);

    // Enable system messages
    system_set_os_print(1);
    os_printf("\r\nSystem init...\r\n");

    // Init pressure sensor
    if (BMP180_Init()) {
        ets_uart_printf("BMP180 init done.\r\n");
    } else {
        ets_uart_printf("BMP180 init error.\r\n");
    }

    if (SHT21_Init()) {
    	ets_uart_printf("SHT21 init done.\r\n");
    }
    else
    	ets_uart_printf("SHT21 init error.\r\n");

    if(wifi_get_opmode() != STATION_MODE) {
        os_printf("ESP8266 is %s mode, restarting in %s mode...\r\n", WiFiMode[wifi_get_opmode()], WiFiMode[STATION_MODE]);
        setup_wifi_st_mode();
    }
    if(wifi_get_phy_mode() != PHY_MODE_11N)
        wifi_set_phy_mode(PHY_MODE_11N);
    if(wifi_station_get_auto_connect() == 0)
        wifi_station_set_auto_connect(1);

    // Wait for Wi-Fi connection
    os_timer_disarm(&WiFiLinker);
    os_timer_setfn(&WiFiLinker, (os_timer_func_t *)wifi_check_ip, NULL);
    os_timer_arm(&WiFiLinker, 5000, 0);

    // Set up a timer to send the message
    os_timer_disarm(&measure_timer);
    os_timer_setfn(&measure_timer, (os_timer_func_t *)measure_cb, (void *)0);
    os_timer_arm(&measure_timer, DATA_SEND_DELAY, 1);

    os_printf("System init done.\n");
}
